shared float4x4 World;
shared float4x4 View;
shared float4x4 Projection;
shared float3 cameraPosition;

//light properties
shared float3 lightPosition;
shared float4 ambientLightColor;
shared float4 diffuseLightColor;
shared float4 specularLightColor;

//material properties
shared float specularPower;
shared float specularIntensity;

struct VS_Output
{
     float4 Position : POSITION;
     float3 WorldNormal : TEXCOORD0;
     float3 WorldPosition : TEXCOORD1;
};

VS_Output PerPixelDiffuseVS(
     float3 position : POSITION,
     float3 normal : NORMAL )
{
     VS_Output output;

     //generate the World-View-Projection matrix
     float4x4 wvp = mul(mul(World, View), Projection);
     
     //transform the input position to the output
     output.Position = mul(float4(position, 1.0), wvp);

     output.WorldNormal =  mul(normal, World);
     float4 worldPosition =  mul(float4(position, 1.0), World);
     output.WorldPosition = worldPosition / worldPosition.w;

     //return the output structure
     return output;
}

float4 DiffuseAndPhongPS(VS_Output input) : COLOR
{
//	if (dot(cameraPosition - input.WorldPosition, cameraPosition - input.WorldPosition) <= 4.5*4.5)
//		discard;
     //calculate per-pixel diffuse
     float3 directionToLight = normalize(lightPosition - input.WorldPosition);
     float diffuseIntensity = saturate( dot(directionToLight, input.WorldNormal));
     float4 diffuse = diffuseLightColor * diffuseIntensity;

     //calculate Phong components per-pixel
     float3 reflectionVector = normalize(reflect(-directionToLight, input.WorldNormal));
     float3 directionToCamera = normalize(cameraPosition - input.WorldPosition);
     
     //calculate specular component
     float4 specular = specularLightColor * specularIntensity * 
                       pow(saturate(dot(reflectionVector, directionToCamera)), 
                           specularPower);
      
     //all color components are summed in the pixel shader
     float4 color = specular  + diffuse + ambientLightColor;
     color.a = 1.0;
     return color;
}

technique PerPixelDiffuseAndPhong
{
     
    pass P0
    {
          //set the VertexShader state to the basic
          //vertex shader that will set up inputs for
          //the pixel shader
          VertexShader = compile vs_2_0 PerPixelDiffuseVS();
          
          //set the PixelShader state to the complete Phong Shading
          //implementation of the pixel shader       
          PixelShader = compile ps_2_0 DiffuseAndPhongPS();
    }
}